package com.jclark.xsl;

class PatternTokenizer {
  private String pattern;
  private Node node;
  private int nextIndex = 0;
  private int patternLength;
  private String current = null;

  PatternTokenizer(Node node, String pattern) {
    this.node = node;
    this.pattern = pattern;
    patternLength = pattern.length();
  }

  Node getNode() {
    return node;
  }

  Name expandElementTypeName(String name) throws XSLException {
    return node.getNamespacePrefixMap().expandElementTypeName(name, node);
  }

  Name expandAttributeName(String name) throws XSLException {
    return node.getNamespacePrefixMap().expandAttributeName(name, node);
  }

  static public final boolean isFunction(String str) {
    return str != null && str.charAt(str.length() - 1) == '(';
  }

  static public final boolean isName(String str) {
    return (str != null
	    && !isDelimiter(str.charAt(0))
	    && str.charAt(0) != '.'
	    && str.charAt(str.length() - 1) != '(');
  }

  static private final boolean isDelimiter(char c) {
    return " \t\r\n\"'[](),*/=|".indexOf(c) >= 0;
  }

  static private final boolean isQuote(char c) {
    return c == '\'' || c == '"';
  }

  static private final boolean isWhitespace(char c) {
    return " \t\r\n".indexOf(c) >= 0;
  }

  String getCurrent() {
    return current;
  }

  // Treat "name(" as a single token.

  String next() throws XSLException {
    char c;
    for (;;) {
      if (nextIndex >= patternLength)
	return current = null;
      c = pattern.charAt(nextIndex);
      if (!isWhitespace(c))
	break;
      nextIndex++;
    }
    int startIndex = nextIndex++;
    if (isQuote(c)) {
      nextIndex = pattern.indexOf(c, nextIndex);
      if (nextIndex < 0)
	throw new XSLException("missing quote in pattern", node);
      ++nextIndex;
    }
    else if (isDelimiter(c)) {
      if (c == '/'
	  && nextIndex < patternLength
	  && pattern.charAt(nextIndex) == '/')
	++nextIndex;
    }
    else if (c == '.') {
      if (c == '.'
	  && nextIndex < patternLength
	  && pattern.charAt(nextIndex) == '.')
	++nextIndex;
    }
    else {
      while (nextIndex < patternLength
	     && !isDelimiter(pattern.charAt(nextIndex)))
	nextIndex++;
      for (int i = nextIndex; i < patternLength; i++) {
	if (pattern.charAt(i) == '(') {
	  if (i == nextIndex) {
	    nextIndex++;
	    break;
	  }
	  current = pattern.substring(startIndex, nextIndex) + '(';
	  nextIndex = i + 1;
	  return current;
	}
	if (!isWhitespace(pattern.charAt(i)))
	  break;
      }
    }
    return current = pattern.substring(startIndex, nextIndex);
  }
}
